//
//  TransportModes.m
//  TripPlanner
//
//  Created by Adrian Schoenig on 26/09/12.
//
//

#if SWIFT_PACKAGE
#import <TripKitObjc/TKTransportModes.h>
#import <TripKitObjc/TKStyleManager.h>
#else
#import "TKTransportModes.h"
#import "TKStyleManager.h"
#endif

// all of these need to be unique
static NSString* const kSGTransportModeTypeIdentifierFlight         = @"air";
static NSString* const kSGTransportModeTypeIdentifierRegularPublic  = @"pub";
static NSString* const kSGTransportModeTypeIdentifierLimitedTransit = @"ltd";
static NSString* const kSGTransportModeTypeIdentifierOnDemand       = @"drt";
static NSString* const kSGTransportModeTypeIdentifierTaxi           = @"tax";
static NSString* const kSGTransportModeTypeIdentifierTNC            = @"tnc";
static NSString* const kSGTransportModeTypeIdentifierAutoRickshaw   = @"ars";
static NSString* const kSGTransportModeTypeIdentifierShuttles       = @"shu";
static NSString* const kSGTransportModeTypeIdentifierCar            = @"car";
static NSString* const kSGTransportModeTypeIdentifierCarShare       = @"car-s";
static NSString* const kSGTransportModeTypeIdentifierCarRental      = @"car-r";
static NSString* const kSGTransportModeTypeIdentifierCarPool        = @"car-p";
static NSString* const kSGTransportModeTypeIdentifierMotorbike      = @"mot";
static NSString* const kSGTransportModeTypeIdentifierBicycle        = @"bic";
static NSString* const kSGTransportModeTypeIdentifierBicycleShare   = @"bic-s";
static NSString* const kSGTransportModeTypeIdentifierWalking        = @"wal";
static NSString* const kSGTransportModeTypeIdentifierWheelchair     = @"whe";

NSString *const TKTransportModeIdentifierFlight                    = @"in_air";
NSString *const TKTransportModeIdentifierRegularPublicTransport    = @"pt_pub";
NSString *const TKTransportModeIdentifierSchoolBuses               = @"pt_ltd_SCHOOLBUS";
NSString *const TKTransportModeIdentifierOnDemandTransit           = @"ps_drt";
NSString *const TKTransportModeIdentifierTaxi                      = @"ps_tax";
NSString *const TKTransportModeIdentifierAutoRickshaw              = @"ps_ars";
NSString *const TKTransportModeIdentifierCar                       = @"me_car";
NSString *const TKTransportModeIdentifierMotorbike                 = @"me_mot";
NSString *const TKTransportModeIdentifierBicycle                   = @"me_mic_bic";
NSString *const TKTransportModeIdentifierBikeShare                 = @"me_mic-s_bic";
NSString *const TKTransportModeIdentifierDeprecatedBicycle         = @"cy_bic";
NSString *const TKTransportModeIdentifierDeprecatedBikeShare       = @"cy_bic-s";
NSString *const TKTransportModeIdentifierWalking                   = @"wa_wal";
NSString *const TKTransportModeIdentifierWheelchair                = @"wa_whe";

@implementation TKTransportModes

#pragma mark - Transport mode identifiers

+ (NSArray *)defaultModeIdentifiers
{
  return @[
           TKTransportModeIdentifierRegularPublicTransport,
           TKTransportModeIdentifierTaxi,
           TKTransportModeIdentifierCar,
           TKTransportModeIdentifierMotorbike,
           TKTransportModeIdentifierBicycle,
           TKTransportModeIdentifierWalking,
           ];
}

+ (NSString *)modeImageNameForModeIdentifier:(NSString *)modeIdentifier
{
  NSString *typeString = [self modeTypeIdentifier:modeIdentifier];
  if ([typeString isEqualToString:kSGTransportModeTypeIdentifierRegularPublic]) {
    return @"public-transport";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierOnDemand]) {
    return @"shuttlebus";
  } else if ([modeIdentifier isEqualToString:TKTransportModeIdentifierBicycle] || [modeIdentifier isEqualToString:TKTransportModeIdentifierDeprecatedBicycle]) {
    return @"bicycle";
  } else if ([modeIdentifier isEqualToString:TKTransportModeIdentifierBikeShare] || [modeIdentifier isEqualToString:TKTransportModeIdentifierDeprecatedBikeShare]) {
    return @"bicycle-share";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierCar]) {
    return @"car";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierCarPool]) {
    return @"car-pool";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierCarShare]
             || [typeString isEqualToString:kSGTransportModeTypeIdentifierCarRental]) {
    return @"car-share";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierMotorbike]) {
    return @"motorbike";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierTaxi]) {
    return @"taxi";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierTNC]) {
    return @"car-ride-share";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierAutoRickshaw]) {
    return @"auto-rickshaw";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierShuttles]) {
    return @"shuttlebus";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierWalking]) {
    return @"walk";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierWheelchair]) {
    return @"wheelchair";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierFlight]) {
    return @"aeroplane";
  } else if ([typeString isEqualToString:kSGTransportModeTypeIdentifierLimitedTransit]) {
    return @"bus";
  } else {
    // fall back to generic type
    if ([modeIdentifier hasPrefix:@"pt_"]) {
      return @"bus";
    } else if ([modeIdentifier hasPrefix:@"ps_"]) {
      return @"taxi";
    } else if ([modeIdentifier hasPrefix:@"me_"]) {
      return @"car";
    } else if ([modeIdentifier hasPrefix:@"cy_"]) {
      return @"bicycle";
    } else if ([modeIdentifier hasPrefix:@"wa_"]) {
      return @"walk";
    } else if ([modeIdentifier hasPrefix:@"in_"]) {
      return @"aeroplane";
    } else {
      return nil; // probably a stationary ID
    }
  }
}

+ (NSString *)modeTypeIdentifier:(NSString *)modeIdentifier
{
  NSArray *components = [modeIdentifier componentsSeparatedByString:@"_"];
  if (components.count > 1) {
    return components[1];
  } else {
    return nil;
  }
}

+ (NSString *)genericModeIdentifierForModeIdentifier:(NSString *)modeIdentifier
{
  NSArray *components = [modeIdentifier componentsSeparatedByString:@"_"];
  if (components.count > 1) {
    return [NSString stringWithFormat:@"%@_%@", components[0], components[1]];
  } else {
    return modeIdentifier;
  }
}

+ (BOOL)modeIdentifierIsPublicTransport:(NSString *)modeIdentifier
{
  return [modeIdentifier hasPrefix:@"pt_"];
}

+ (BOOL)modeIdentifierIsCycling:(NSString *)modeIdentifier
{
  if (! modeIdentifier)
    return NO;
  
  NSString *typeIdentifier = [self modeTypeIdentifier:modeIdentifier];
  return [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierBicycle]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierBicycleShare];
}

+ (BOOL)modeIdentifierIsDriving:(NSString *)modeIdentifier
{
  if (! modeIdentifier)
    return NO;
  
  NSString *typeIdentifier = [self modeTypeIdentifier:modeIdentifier];
  return [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCar]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarShare]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarRental]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierMotorbike];
}


+ (BOOL)modeIdentifierIsWalking:(NSString *)modeIdentifier
{
  return [modeIdentifier hasPrefix:@"wa_"];
}

+ (BOOL)modeIdentifierIsWheelchair:(NSString *)modeIdentifier
{
  return [modeIdentifier isEqualToString:TKTransportModeIdentifierWheelchair];
}

+ (BOOL)modeIdentifierIsSharedVehicle:(NSString *)modeIdentifier
{
  NSString *typeIdentifier = [self modeTypeIdentifier:modeIdentifier];
  return [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarShare]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarRental]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierBicycleShare];
}

+ (BOOL)modeIdentifierIsAffectedByTraffic:(NSString *)modeIdentifier
{
  if (! modeIdentifier)
    return NO;
  
  return [modeIdentifier hasPrefix:@"me_"] || [modeIdentifier hasPrefix:@"ps_"];
}

+ (BOOL)modeIdentifierIsExpensive:(NSString *)modeIdentifier
{
  if (! modeIdentifier)
    return NO;
  
  NSString *typeIdentifier = [self modeTypeIdentifier:modeIdentifier];
  return [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarShare]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierCarRental]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierFlight]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierShuttles]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierTaxi]
      || [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierTNC];
}

+ (BOOL)modeIdentifierIsFlight:(NSString *)modeIdentifier
{
  NSString *typeIdentifier = [self modeTypeIdentifier:modeIdentifier];
  return [typeIdentifier isEqualToString:kSGTransportModeTypeIdentifierFlight];
}

@end
