//
//  TKUIRoutingQueryInputViewModel.swift
//  TripKitUI-iOS
//
//  Created by Adrian Schönig on 22.10.19.
//  Copyright © 2019 SkedGo Pty Ltd. All rights reserved.
//

import Foundation
import MapKit

import RxSwift
import RxCocoa

import TripKit

class TKUIRoutingQueryInputViewModel {
  
  typealias Item = TKUIAutocompletionViewModel.Item
  typealias Section = TKUIAutocompletionViewModel.Section

  struct UIInput {
    let searchText: Observable<(String, forced: Bool)>
    let tappedRoute: Signal<Void>
    var tappedKeyboardDone: Signal<TKUIRoutingResultsViewModel.SearchMode> = .empty()
    var selected: Signal<Item> = .empty()
    var selectedSearchMode: Signal<TKUIRoutingResultsViewModel.SearchMode> = .empty()
    var tappedSwap: Signal<Void> = .empty()
  }
  
  init(origin: MKAnnotation? = nil, destination: MKAnnotation? = nil, biasMapRect: MKMapRect = .null, startMode: TKUIRoutingResultsViewModel.SearchMode? = nil, inputs: UIInput, providers: [TKAutocompleting]? = nil) {

    let includeCurrentLocation = TKLocationManager.shared.featureIsAvailable()
    let start: MKAnnotation?
    if let provided = origin {
      start = provided
    } else if includeCurrentLocation {
      start = TKLocationManager.shared.currentLocation
    } else {
      start = nil
    }
    
    let providers = providers ?? TKUIRoutingResultsCard.config.autocompletionDataProviders
    
    let autocompletionModel = TKUIAutocompletionViewModel(
      providers: providers,
      includeCurrentLocation: includeCurrentLocation,
      searchText: inputs.searchText.startWith(("", forced: false)),
      selected: inputs.selected,
      biasMapRect: .just(biasMapRect)
    )
    
    let state = Self.buildState(
        origin: start, destination: destination,
        startMode: startMode,
        inputs: inputs, selection: autocompletionModel.selection
      )
      .share(replay: 1, scope: .whileConnected)
    
    activeMode = state.map { $0.mode }
      .distinctUntilChanged()
      .asDriver(onErrorDriveWith: .empty())
    
    originDestination = state.map { ($0.originText, $0.destinationText) }
      .distinctUntilChanged { $0.0 == $1.0 && $0.1 == $1.1 }
      .asDriver(onErrorDriveWith: .empty())

    sections = autocompletionModel.sections

    triggerAction = autocompletionModel.triggerAction
    
    enableRouteButton = state
      .map { $0.origin != nil && $0.destination != nil }
      .asDriver(onErrorJustReturn: false)

    selections = inputs.tappedRoute.asObservable()
      .withLatestFrom(state)
      .filter { $0.origin != nil && $0.destination != nil }
      .map { ($0.origin!, $0.destination!) }
      .asSignal(onErrorSignalWith: .empty())
}
  
  let activeMode: Driver<TKUIRoutingResultsViewModel.SearchMode?>
  
  let originDestination: Driver<(origin: String, destination: String)>
  
  let sections: Driver<[Section]>
  
  let enableRouteButton: Driver<Bool>
  
  let selections: Signal<(MKAnnotation, MKAnnotation)>
  
  let triggerAction: Signal<TKAutocompleting>
}
